/* 
 * EDOMI beta (Version 1.50) - Intelligente Steuerung und Visualisierung KNX-basierter Elektro-Installationen.
 * @link http://www.edomi.de 
 * @author Dr. Christian Gärtner <dr.christian.gaertner@googlemail.com> 
 * @copyright 2015,2016,2017 Dr. Christian Gärtner 
 * @license Freeware 
 *
 *     This file and all other files are part of EDOMI.
 *
 *     EDOMI is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *     Diese und alle anderen Dateien sind Teil von EDOMI.
 *
 *     EDOMI wird in der Hoffnung, dass es nützlich sein wird, aber
 *     OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *     Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
*/ 
/*
============================================================================
Controls (alle Apps)
============================================================================
*/

function controlGetFormData(formId) {
	//bildet aus allen Controls in dem "Formular" formId einen String ("1;2;3;...;")
	//die Controls müssen alle eine ID in der Art "winId-fd#" haben, wobei # eine laufende Nummer ist (fd1,fd2,...)
	//die Reihenfolge ist egal, aus fd1=1,fd3=3,fd2=2 wird "1;2;3;" (NICHT "1;3;2;")
	//fehlende Nummern werden im String aufgefüllt: aus fd1=1,fd2=2,fd4=4 wird "1;2;;4;"
	var n=new Array();
	var tmp=document.getElementById(formId).querySelectorAll("[data-type]");
	for (var t=0; t<tmp.length; t++) {
		var type=tmp[t].dataset.type;
		if (tmp[t].id) {var id=tmp[t].id.split("-")[1].replace("fd","");}

		//0: Dummy-Control (dient nur der Übertragung per Formular)
		if (type==0)  {n[id]=stringCleanup(tmp[t].dataset.value);}		//Dummy-Control

		//1..9: Eigene Input-Controls (ohne DB usw.)
		//---------------------------------------------------------------------------------
		if (type==1)  {n[id]=stringCleanup(tmp[t].value);}				//Input/Textarea
		if (type==5)  {n[id]=stringCleanup(tmp[t].dataset.value);}		//Checkbox
		if (type==6)  {n[id]=stringCleanup(tmp[t].dataset.value);}		//Selectbox
		if (type==9)  {n[id]=stringCleanup(tmp[t].dataset.value);}		//Folder-Select

		//10..999: editRoot-Verweise
		//---------------------------------------------------------------------------------
		if (type>=10 && type<=999) {n[id]=stringCleanup(tmp[t].dataset.value);}
		//11: LogicPage-Select
		//12: LBS-Select
		//21: Visu-Select
		//22: VisuPage-Select
		//23: VisuUser-Select
		//24: VisuElementDef-Select
		//25: Visu-BG-Color-Select
		//26: Visu-FG-Color-Select
		//27: VisuAnimation-Select
		//28: VisuImage-Select
		//29: VisuSound-Select
		//30: GA-Select
		//40: Szene-Select
		//81: Cam-Select
		//82: CamArchiv-Select
		//50: Datenarchiv-Select
		//60: Meldungsarchiv-Select
		//70: IP-Telegramm-Select
		//75: IR-Befehl-Select
		//90: Sequenz-Select
		//100: Zeitschaltuhr-Select
		//110: AWS-Select
		//120: eMail-Select
		//125: Telefon-Select
		//127: Anrufarchiv-Select
		//130: Diagramm-Select

		//1000: -
		//1001: Szenen-Liste
		//1002: VisuElement-Edit
		//1003: Style-Liste (Visu-Element)
		//1004: (Server-Uhrzeit)
		//1005: Werteingabe (z.B. Fixwert in Logic)
		//1006: Aktion: Befehle (für Logic-Ausgang)
		//1007: Befehle-Liste (universell! dataset.db = ZielDB)
		//1008: Diagramm-Liste
		//1009: AWS-Liste
		//1010: Live-Wert setzen (Logikeditor)
		//1011: frei
		//1012: frei
		//1013: Datum/Uhrzeit für Server setzen
		//1014: Kameraarchivbild (für app102)
		//1015: IR-Trans lernen
		//1016: (Hilfe)
		//1017: VisuUser-Liste
		//1019: ZSU-Daten
		//1022: TSU-Daten
	}
	var r="";
	for (var t=0; t<n.length; t++) {
		if (n[t]==undefined) {n[t]="";}
		r+=n[t]+AJAX_SEPARATOR1;
	}
	return r;
}

function controlInitAll(parentId) {
	//Initialisiert alle Controls, die Childs von parentId sind (z.B. in einem Formular)
	if (document.getElementById(parentId)) {
		var tmp=document.getElementById(parentId).querySelectorAll("[data-type]");

		//bulk-Verarbeitung (Ajax-Controls)
		var r="";
		for (var t=0; t<tmp.length; t++) {
			if (tmp[t].dataset.type>0)  {r+=controlInit(tmp[t].id,true);}
		}
		if (r!="") {ajax("initControls",1000,"",r,"");}

/*
		//einzelne Ajax-Calls für jedes Control (Ajax-Controls)
		for (var t=0; t<tmp.length; t++) {
			if (tmp[t].dataset.type>0)  {controlInit(tmp[t].id);}
		}
*/
	}
}

function controlInit(senderId,bulk) {
	//Initialisiert das Control senderId
	//senderId: das aufzubauende Control
	//bulk (OPTIONAL): true=Ajax-Requests gesammelt zurückgeben
	var senderObj=document.getElementById(senderId);
	var r="";
	if (bulk!==true) {bulk=false;}

	//1..9: Eigene Input-Controls (ohne DB usw.)
	//---------------------------------------------------------------------------------
	if (senderObj.dataset.type==5) { //Checkbox
		if (senderObj.dataset.value==1) {
			var options=senderObj.dataset.options;
			if (options=='-warning') {
				senderObj.style.background="#e00000";
				senderObj.style.color="#ffffff";
			} else {
				senderObj.style.background="#80e000";
				senderObj.style.color="";
			}
		} else {
			senderObj.style.background="#d0d0d0";
			senderObj.style.color="#909090";
		}
		if (!senderObj.getAttribute("onMouseDown")) { //nur zur Sicherheit...
			senderObj.setAttribute("onMouseDown","controlClick('"+senderId+"');");
		}
	}

	if (senderObj.dataset.type==6) { //Selectbox
		var n="";
		var id=senderObj.dataset.value;
		var list=senderObj.dataset.list.split(";");
		var options=senderObj.dataset.options;

		controlChange(senderId);

		if (options=='-noEdit') {
			n+="<select class='control6select'>";
			for (var t=0;t<(list.length-1);t++) {
				if (id==list[t].split("|")[0]) {
					n+="<option value='"+list[t].split("|")[0]+"' selected>"+list[t].split("|")[1]+"</option>";
				}
			}
			n+="</select>";
		} else {
			n+="<select onChange='document.getElementById(\""+senderId+"\").dataset.value=this.value; controlChange(\""+senderId+"\");' class='control6select'>";
			for (var t=0;t<(list.length-1);t++) {
				if (id==list[t].split("|")[0]) {
					n+="<option value='"+list[t].split("|")[0]+"' selected>"+list[t].split("|")[1]+"</option>";
				} else {
					n+="<option value='"+list[t].split("|")[0]+"'>"+list[t].split("|")[1]+"</option>";
				}
			}
			n+="</select>";
		}
		senderObj.innerHTML=n;
	}

	//10..999: editRoot-Verweise
	//---------------------------------------------------------------------------------
	if (senderObj.dataset.type>=10 && senderObj.dataset.type<=999) {
		if (bulk) {
			r="initControl-"+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
			senderObj.setAttribute("onMouseDown","controlClick('"+senderId+"');");
		} else {
			ajax("initControl-"+senderObj.dataset.type,1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options,"");
			senderObj.setAttribute("onMouseDown","controlClick('"+senderId+"');");
		}
	}

	//1000..9999: Spezielle Controls (z.B. LogicCmd, Listen-Items, etc.)
	//---------------------------------------------------------------------------------
	if (senderObj.dataset.type==1001) { //Szenen-Liste
		if (bulk) {
			r="initControl-1001"+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
		} else {
			ajax("initControl-1001",1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid,"");
		}
	}
	if (senderObj.dataset.type==1002) { //VisuElement-Edit
		if (bulk) {
			r="initControl-1002"+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
		} else {
			ajax("initControl-1002",1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options,"");
		}
	}
	if (senderObj.dataset.type==1003) { //Style-Liste (Visu)
		if (bulk) {
			r="initControl-1003"+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
		} else {
			ajax("initControl-1003",1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid,"");
		}
	}
	if (senderObj.dataset.type==1007) { //Befehle-Liste
		if (bulk) {
			r="initControl-1007"+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
		} else {
			ajax("initControl-1007",1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db,"");
		}
	}
	if (senderObj.dataset.type==1008) { //Diagramm-Liste
		if (bulk) {
			r="initControl-1008"+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
		} else {
			ajax("initControl-1008",1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db,"");
		}
	}
	if (senderObj.dataset.type==1009) { //AWS-Liste
		if (bulk) {
			r="initControl-1009"+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
		} else {
			ajax("initControl-1009",1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db,"");
		}
	}
	if (senderObj.dataset.type==1017) { //VisuUser-Liste
		if (bulk) {
			r="initControl-1017"+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
		} else {
			ajax("initControl-1017",1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid,"");
		}
	}
	if (senderObj.dataset.type==1019) { //ZSU-Daten
		if (bulk) {
			r="initControl-1019"+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
		} else {
			ajax("initControl-1019",1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db,"");
		}
	}
	if (senderObj.dataset.type==1022) { //TSU-Daten
		if (bulk) {
			r="initControl-1022"+AJAX_SEPARATOR1+senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db+AJAX_SEPARATOR1+""+AJAX_SEPARATOR2;
		} else {
			ajax("initControl-1022",1000,"",senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db,"");
		}
	}
	return r;
}

function controlChange(senderId) {
	//ggf. diverse Änderungen beim onChange eines Controls
	//senderId: das angeklickte Control
	var senderObj=document.getElementById(senderId);

	if (senderObj.dataset.type==6) { //Selectbox
		//DOM-Elemente ggf. aus-/einblenden
		var id=senderObj.dataset.value;
		var list=senderObj.dataset.list.split(";");
		for (var t=0;t<(list.length-1);t++) {
			var radioId=list[t].split("|")[2];
			if (radioId) {
				if (id==list[t].split("|")[0]) {
					if (radioId.search("radiotr")>=0) {
						document.getElementById(radioId).style.display="table-row";
					} else {
						document.getElementById(radioId).style.display="inline";
					}
				} else {
					document.getElementById(radioId).style.display="none";
				}
			}
		}
	}
}

function controlClick(senderId) {
	//senderId: das angeklickte Control
	if (!event) var event=window.event;
	if (event.button==0) {controlClickLeft(senderId);}
	if (event.button==2) {controlClickRight(senderId);}
	clickCancel();
}

function controlClickLeft(senderId) {
	var senderObj=document.getElementById(senderId);

	//1..9: Eigene Input-Controls (ohne DB usw.)
	//---------------------------------------------------------------------------------
	if (senderObj.dataset.type==5) { //Checkbox
		if (senderObj.dataset.value==1) {senderObj.dataset.value=0;} else {senderObj.dataset.value=1;}
		controlInit(senderObj.id);
	}

	//10..999: editRoot-Verweise
	//---------------------------------------------------------------------------------
	if (senderObj.dataset.type>=10 && senderObj.dataset.type<=999) {
		if (!(senderObj.dataset.value>0) && app1000_history["id"+senderObj.dataset.type]>0) {
			openWindow(1000,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+app1000_history["id"+senderObj.dataset.type]);
		} else {
			openWindow(1000,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options);
		}
	}

	//1000..9999: Spezielle Controls (z.B. LogicCmd, Listen-Items, etc.)
	//---------------------------------------------------------------------------------
	if (senderObj.dataset.type==1001) {openWindow(1001,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid);} //Szenen-Liste (controlList)
	if (senderObj.dataset.type==1002) {openWindow(1002,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options);} //VisuElement-Edit
	if (senderObj.dataset.type==1003) {openWindow(1003,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid);} //Style-Liste (Visu)
	if (senderObj.dataset.type==1005) {openWindow(1005,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options);} //Werteingabe (z.B. Fixwert in Logic)
	if (senderObj.dataset.type==1006) {openWindow(1006,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db);} //Aktion: Befehle (für Logic)
	if (senderObj.dataset.type==1007) {openWindow(1007,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db);} //Befehle-Liste (controlList)
	if (senderObj.dataset.type==1008) {openWindow(1008,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid+AJAX_SEPARATOR1+senderObj.dataset.db);} //Diagramm-Liste (controlList)
	if (senderObj.dataset.type==1009) {openWindow(1009,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid);} //AWS-Liste (controlList)
	if (senderObj.dataset.type==1017) {openWindow(1017,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid);} //VisuUser-Liste (controlList)
	if (senderObj.dataset.type==1019) {openWindow(1019,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid);} //ZSU-Daten (controlList)
	if (senderObj.dataset.type==1022) {openWindow(1022,senderObj.id+AJAX_SEPARATOR1+senderObj.dataset.type+AJAX_SEPARATOR1+senderObj.dataset.value+AJAX_SEPARATOR1+senderObj.dataset.options+AJAX_SEPARATOR1+senderObj.dataset.itemid);} //TSU-Daten (controlList)
}

function controlClickRight(senderId) {
	var senderObj=document.getElementById(senderId);
	var winId=senderId.split("-")[0]; //für Kontextmenü

	//controlList? (dann die ID des angeklickten Items zusammenbasteln)
	var itemObj=document.getElementById(senderId+"-"+senderObj.dataset.itemid);
	if (itemObj) {
		if (itemObj.dataset.cm0 || itemObj.dataset.cm1 || itemObj.dataset.cm2 || itemObj.dataset.cm3) {
			var winId=senderId.split("-")[0];
			var cm=controlContextmenuOpen(winId);
			var n='';
			if (itemObj.dataset.cm1) {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); "+itemObj.dataset.cm1+"'>nach oben rücken</div>";}
			if (itemObj.dataset.cm2) {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); "+itemObj.dataset.cm2+"'>nach unten rücken</div>";}
			if (itemObj.dataset.cm3) {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); "+itemObj.dataset.cm3+"'>Duplizieren</div>";}
			if (itemObj.dataset.cm0) {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); "+itemObj.dataset.cm0+"'>Löschen</div>";}
			cm.innerHTML=n;
		}
	}
}

function controlReturn(winID,senderId,value) {
	//winID: winID des Popup-Fensters
	//senderId: Id des Controls, das das Popup aufgerufen hat
	//value: der zu übernehmende Wert
	var senderObj=document.getElementById(senderId);
	if (senderObj) {
		senderObj.dataset.value=value;
		if (!(senderObj.dataset.options && senderObj.dataset.options.search('-noRefresh')>=0)) {controlInit(senderId);}
		if (senderObj.dataset.callback) {eval(senderObj.dataset.callback);}
	}
	if (document.getElementById(winID)) {closeWindow(winID)};
}

function controlCancel(winID,senderId) {
	//wie controlReturn, nur wird value nicht verändert (Control wird refreshed und callback2(!) ggf. aufgerufen)
	//winID: winID des Popup-Fensters
	//senderId: Id des Controls, das das Popup aufgerufen hat
	var senderObj=document.getElementById(senderId);
	if (senderObj) {
		if (!(senderObj.dataset.options && senderObj.dataset.options.search('-noRefresh')>=0)) {controlInit(senderId);}
		if (senderObj.dataset.callback2) {eval(senderObj.dataset.callback2);}
	}
	if (document.getElementById(winID)) {closeWindow(winID)};
}


function controlContextmenuOpen(winId) {
if (document.getElementById(winId+'-contextmenu')) {controlContextmenuClose(winId);}
	if (!event) {var event=window.event;}
	var pobj=document.getElementById(winId); //winId IST das parent-Objekt... (wegen absoluter Mausposition)
	var cm=createNewDiv(winId,winId+'-contextmenu');
	cm.className='contextMenu';
	cm.style.left=(parseInt(event.pageX)-15)+'px';
	cm.style.top=(parseInt(event.pageY)-15)+'px';
//	cm.setAttribute('onMouseOut','if (this==event.target) {controlContextmenuClose("'+winId+'");}');
//	cm.setAttribute('onMouseOut','if (event.target.id=="'+winId+'-contextmenu") {controlContextmenuClose("'+winId+'");}');
	cm.setAttribute('onMouseLeave','controlContextmenuClose("'+winId+'");');
	return cm;
}

function controlContextmenuClose(winId) {
	var tmp=document.getElementById(winId+'-contextmenu');
	if (tmp) {tmp.parentNode.removeChild(tmp);}
}


/*
============================================================================
Ordner und Items (App 1000)
============================================================================
*/

function app1000_folderClick(winId,objId,folderId) {
	if (!event) var event=window.event;
	if (event.button==0) {app1000_folderClickLeft(winId,objId,folderId);}
	if (event.button==2) {app1000_folderClickRight(winId,objId);}
	clickCancel();
}

function app1000_folderClickLeft(winId,objId,folderId) {
	var obj=document.getElementById(objId);
	app1000_folderSelect(winId,objId);
	clickCancel();
	if (folderId!='') {
		app1000_folderToggle(winId,folderId);
	}
}

function app1000_folderClickRight(winId,objId) {
	//Item-Data: item.id, startFolderId, folder.id, folder.allow, returnAllowed, senderId

	app1000_folderSelect(winId,objId);

	var obj=document.getElementById(objId);
	var globalVars=document.getElementById(winId+"-globalVars");

	if (globalVars && obj.dataset.itemdata) {
		var itemData=obj.dataset.itemdata.split(AJAX_SEPARATOR1);
		var n='';

		if (itemData[3].substr(0,1)=="X") {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"newFolder\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+"\");'>Neuer Ordner</div>";}
		if (itemData[3].substr(1,1)=="X") {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"renameFolder\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+"\");'>Ordner umbennenen</div>";}
		if (itemData[3].substr(2,1)=="X") {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"cutFolder\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+"\");'>Ordner merken</div>";}
		if (itemData[3].substr(4,1)=="X") {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"duplicateFolder\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+"\");'>Ordner duplizieren</div>";} //noch nicht implementiert
		if (itemData[3].substr(5,1)=="X") {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"deleteFolder\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+"\");'>Ordner löschen</div>";}
		
		//Paste
		if ((itemData[3].substr(3,1)=="X" && document.getElementById(winId+"-globalVars").dataset.cutfolder>0) || (itemData[3].substr(9,1)=="x" && document.getElementById(winId+"-globalVars").dataset.cutitem>0)) {
			n+="<div style='padding:3px;'><div style='widht:100%; height:1px; background:#c0c0c0;'></div></div>";
		}
		if (itemData[3].substr(3,1)=="X" && document.getElementById(winId+"-globalVars").dataset.cutfolder>0) {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"pasteFolder\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+AJAX_SEPARATOR1+document.getElementById(winId+"-globalVars").dataset.cutfolder+"\");'><b>Ordner hierher verschieben</b></div>";}
		if (itemData[3].substr(9,1)=="x" && document.getElementById(winId+"-globalVars").dataset.cutitem>0) {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"pasteItem\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+AJAX_SEPARATOR1+document.getElementById(winId+"-globalVars").dataset.cutitem+AJAX_SEPARATOR1+document.getElementById(winId+"-globalVars").dataset.cutitemdb+"\");'><b>Element hierher verschieben</b></div>";}

		//KNX-GAs: Benutzung auflisten
		if (itemData[2]==32) {
			n+="<div style='padding:3px;'><div style='widht:100%; height:1px; background:#c0c0c0;'></div></div>";
			n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"showUsedItems\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+"\");'>Genutzte KNX-GAs anzeigen</div>";
			n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"showUnusedItems\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+"\");'>Ungenutzte KNX-GAs anzeigen</div>";
		}

		//Export
		if (n!='') {n+="<div style='padding:3px;'><div style='widht:100%; height:1px; background:#c0c0c0;'></div></div>";}	
		if (app1000_exportCheck(itemData[2],itemData[0])!==false) {
			n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); app1000_exportRemove(\""+itemData[2]+"\",0);'>Ordner ("+itemData[2]+") aus Exportliste entfernen</div>";
		} else {
			n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); app1000_exportAdd(\""+itemData[2]+"\",0);'>Ordner ("+itemData[2]+") &rarr; Exportliste</div>";
		}

		if (n!='') {
			var cm=controlContextmenuOpen(winId);
			cm.innerHTML=n;
		}
	}

	clickCancel();
}

function app1000_folderToggle(winId,objId) {
	var obj=document.getElementById(objId);
	//Ordner ein/ausklappen
	var childs=obj.children;
	for (var t=1; t<childs.length; t++) {
		if (childs[t].style.display!="none") {childs[t].style.display="none";} else {childs[t].style.display="block";}
	}
}

function app1000_folderItemClick(winId,objId) {
	if (!event) var event=window.event;
	if (event.button==0) {app1000_folderItemClickLeft(winId,objId);}
	if (event.button==2) {app1000_folderItemClickRight(winId,objId);}
	clickCancel();
}

function app1000_folderItemClickLeft(winId,objId) {
	app1000_folderSelect(winId,objId);
}

function app1000_folderItemClickRight(winId,objId) {
	//Item-Data: item.id, startFolderId, folder.id, folder.allow, returnAllowed, senderId

	app1000_folderSelect(winId,objId);

	var obj=document.getElementById(objId);
	var globalVars=document.getElementById(winId+"-globalVars");
	
	if (globalVars && obj.dataset.itemdata) {

		var itemData=obj.dataset.itemdata.split(AJAX_SEPARATOR1);	
		var n='';

		if (obj.dataset.help) {
			n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); openWindow(9999,\""+obj.dataset.help+"\",\"\");'>Hilfe</div>";
			n+="<div style='padding:3px;'><div style='widht:100%; height:1px; background:#c0c0c0;'></div></div>";
		}

		n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"showItemLinks\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+AJAX_SEPARATOR1+itemData[0]+"\");'>Verweise anzeigen</div>";

		if (itemData[3].substr(8,1)=="x") {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"cutItem\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+AJAX_SEPARATOR1+itemData[0]+"\");'>Element merken</div>";}
		if (itemData[3].substr(10,1)=="x") {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"duplicateItem\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+AJAX_SEPARATOR1+itemData[0]+"\");'>Element duplizieren</div>";}
		if (itemData[3].substr(11,1)=="x") {n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"deleteItem\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+AJAX_SEPARATOR1+itemData[0]+"\");'>Element löschen</div>";}
		
		//Paste
		if (itemData[3].substr(9,1)=="x" && document.getElementById(winId+"-globalVars").dataset.cutitem>0) {
			n+="<div style='padding:3px;'><div style='widht:100%; height:1px; background:#c0c0c0;'></div></div>";
			n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajax(\"pasteItem\",\"1000\",\""+winId+"\",\""+globalVars.dataset.jsdata+"\",\""+itemData[2]+AJAX_SEPARATOR1+document.getElementById(winId+"-globalVars").dataset.cutitem+AJAX_SEPARATOR1+document.getElementById(winId+"-globalVars").dataset.cutitemdb+"\");'><b>Element hierher verschieben</b></div>";
		}

		if (obj.dataset.imgPreview) {
			n+="<div style='width:120px; height:120px; margin-top:5px; background:url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAQAAAAECAYAAACp8Z5+AAAAJUlEQVQIHWNsaGj4zwAFCgoKDEzInAcPHkAEQDIgDggwIXNAAgCU4wvnT6Hq5QAAAABJRU5ErkJggg==) repeat;'>";
				n+="<div style='width:100%; height:100%; background-size:contain; background-repeat:no-repeat; background-image:url(\""+obj.dataset.imgPreview+"\");'></div>";
			n+="</div>";
		}

		//Export
		if (n!='') {n+="<div style='padding:3px;'><div style='widht:100%; height:1px; background:#c0c0c0;'></div></div>";}	
		if (app1000_exportCheck(itemData[2],itemData[0])!==false) {
			n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); app1000_exportRemove(\""+itemData[2]+"\",\""+itemData[0]+"\");'>Element ("+itemData[0]+") aus Exportliste entfernen</div>";
		} else {
			n+="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); app1000_exportAdd(\""+itemData[2]+"\",\""+itemData[0]+"\");'>Element ("+itemData[0]+") &rarr; Exportliste</div>";
		}

		if (n!='') {
			var cm=controlContextmenuOpen(winId);
			cm.innerHTML=n;
		}
	}

	clickCancel();	
}

function app1000_exportAdd(folderId,itemId) {
	ajax("exportAdd","1023","","",folderId+AJAX_SEPARATOR1+itemId);
}

function app1000_exportRemove(folderId,itemId) {
	ajax("exportRemove","1023","","",folderId+AJAX_SEPARATOR1+itemId);
}

function app1000_exportCheck(folderId,itemId) {
//	ajax("exportCheck","1023","","",folderId+AJAX_SEPARATOR1+itemId);

//### callback irgendwie... (von PHP aus)
//### oder das Kontextmenü muss bereits von PHP generiert werden...

/*
	for (var t=0; t<app1000_export.length; t++) {
		if (app1000_export[t].folderId==folderId && app1000_export[t].itemId==itemId) {return t;}
	}
*/
	return false;
}

function app1000_folderItemSelect(winId,itemObjId) {
	//Item auswählen und alle erforderlichen Ordner aufklappen (rekursiv bis zum Zielordner)
	var i=document.getElementById(itemObjId);
	if (i) {
		app1000_folderSelect(winId,i.id);
		while (i.parentNode.id.search(winId+"-f-")>=0) {
			var childs=i.parentNode.children;
			for (var t=0; t<childs.length; t++) {childs[t].style.display='block';}
			i.parentNode.style.display='block';
			i=i.parentNode;
		}
		document.getElementById(itemObjId).scrollIntoView();
	}
}

function app1000_folderSelect(winId,objId) {
	//Item-Data: item.id, startFolderId, folder.id, folder.allow, returnAllowed, senderId

	var obj=document.getElementById(objId);
	var globalVars=document.getElementById(winId+"-globalVars");
	
	if (globalVars && obj.dataset.itemdata) {

		var itemData=obj.dataset.itemdata.split(AJAX_SEPARATOR1);
	
		var okButton=document.getElementById(winId+"-okButton");
		var editButton=document.getElementById(winId+"-editButton");
		var newButton=document.getElementById(winId+"-newButton");
		
		if (okButton) {
			if (itemData[0]>0 && itemData[4]=="1") {
				okButton.style.color='#000000';
				okButton.dataset.cmd='app1000_folderItemReturn("'+winId+'","'+itemData[5]+'","'+itemData[0]+'","'+itemData[1]+'","'+itemData[2]+'");';
			} else {
				okButton.style.color='#a0a0a0';
				okButton.dataset.cmd='';
			}
		}
	
		if (editButton) {
			if (itemData[0]>0 && itemData[3].substr(7,1)=="x") {
				editButton.style.color='#000000';
				editButton.dataset.cmd='ajax("editItem","1000","'+winId+'","'+globalVars.dataset.jsdata+'","'+itemData[2]+AJAX_SEPARATOR1+itemData[0]+'");';
			} else {
				editButton.style.color='#a0a0a0';
				editButton.dataset.cmd='';
			}	
		}
	
		if (newButton) {
			if (itemData[3].substr(6,1)=="x") {
				newButton.style.color='#000000';
				newButton.dataset.cmd='ajax("newItem","1000","'+winId+'","'+globalVars.dataset.jsdata+'","'+itemData[2]+AJAX_SEPARATOR1+'-1");';
			} else {
				newButton.style.color='#a0a0a0';
				newButton.dataset.cmd='';
			}
		}

	}
	
	
	//Markierung löschen und auf das aktuelle item setzen
	var tmp=document.getElementById(winId+"-f-0").getElementsByTagName("div")
	for (var t=0; t<tmp.length; t++) {
		tmp[t].style.background="";
	}
	obj.style.background="#80e000";
}

function app1000_folderSearchItem(winId,data) {
	ajax("searchItem","1000",winId,data,stringCleanup(document.getElementById(winId+"-searchInput").value));
}

function app1000_folderOpen(winId,folderId) {
	//Ordner aufklappen (rekursiv bis zum Zielordner)
	var f=document.getElementById(winId+"-f-"+folderId);
	if (f) {
		while (f.parentNode.id.search(winId+"-f-")>=0) {
			var childs=f.children;
			for (var t=0; t<childs.length; t++) {
				childs[t].style.display='block';
			}
			f.parentNode.style.display='block';
			f=f.parentNode;
		}
		document.getElementById(winId+"-f-"+folderId).scrollIntoView();
	}
}

function app1000_folderItemReturn(winID,senderId,value,DBstartFolderId,DBfolderId) {
	if (DBstartFolderId>0 && DBstartFolderId<1000) {
		app1000_history["id"+DBstartFolderId]=DBfolderId;
	}
	controlReturn(winID,senderId,value);
}



/*
============================================================================
Animation/Font-Preview (App 1000)
============================================================================
*/

function app1000_animationPreview(objId,animPara,animData) {
	//CSS-Keyframes in HEAD kopieren
	document.getElementById("cssPreview").innerHTML="@-webkit-keyframes animPreviewData {"+animData+"}";
	//Animation starten
	document.getElementById(objId).style.webkitAnimation="";
	document.getElementById(objId).style.webkitAnimationPlayState="paused";
	setTimeout(function(){document.getElementById(objId).style.webkitAnimation=animPara; document.getElementById(objId).style.webkitAnimationFillMode="forwards"; document.getElementById(objId).style.webkitAnimationPlayState="running";},100);
}

function app1000_fontPreview(objId,fontName,fontTyp) {
	obj=document.getElementById(objId);
	if (obj) {
		if (fontTyp==1) {
			//CSS in HEAD kopieren
			if (fontName!="") {
				document.getElementById("cssPreview").innerHTML="@font-face {font-family:'PREVIEWFONT'; font-style:normal; font-weight:normal; src:url('../../data/project/visu/etc/"+fontName+"') format('truetype');}";
				obj.style.fontFamily="PREVIEWFONT,EDOMIfont,Arial";
			} else {
				document.getElementById("cssPreview").innerHTML="";
				obj.style.fontFamily="EDOMIfont,Arial";
			}
		} else {
			obj2=document.getElementById(fontName);
			if (obj2) {
				var tmp=obj2.value.split(",")[0];
				obj.style.fontFamily=tmp+",EDOMIfont,Arial";
			}
		}
	}
}


/*
============================================================================
Color-Preview (App 1000)
============================================================================
*/

function app1000_colorBGpreview(objId,colorData,ko) {
	document.getElementById(objId).style.background="transparent";
	if (colorData!='') {document.getElementById(objId).style.background=pS(colorData,ko);}
}

function app1000_colorFGpreview(objId,colorData,ko) {
	document.getElementById(objId).style.color="transparent";
	if (colorData!='') {document.getElementById(objId).style.color=pS(colorData,ko);}
}


/*
============================================================================
TAB-Taste in Textarea durchreichen (App 1000)
============================================================================
*/
function app1000_enableTabKey(obj) {
	//Aufruf: <textarea ... onkeydown='if (event.keyCode==9) {app1000_enableTabKey(this);}' ...>
	if (!event) {var event=window.event;}
	event.preventDefault();
	
	var val=obj.value,
	start=obj.selectionStart,
	end=obj.selectionEnd;
	obj.value=val.substring(0,start)+'\t'+val.substring(end);
	obj.selectionStart=obj.selectionEnd=start+1;

	return false;
}

/*
============================================================================
Textarea-Value zurückgeben (App 1018)
============================================================================
*/
function app1018_returnSource(winId,sourceId,destId) {
	document.getElementById(destId).value=document.getElementById(sourceId).value;
	closeWindow(winId);
}

/*
============================================================================
Styles (App 1003)
============================================================================
*/

function app1003_resetForm(winId) {
	//leert alle Eingabefelder
	for (var t=1;t<=48;t++) {
		var obj=document.getElementById(winId+"-fd"+parseInt(t+10));
		if (obj) {
			if (obj.dataset.type<=1) {
				obj.value="";
			} else {
				obj.dataset.value="";
			}
		}
	}
	controlInitAll(winId+"-form1");
}

function app1003_setDefaultStyle(objId,newValue) {
	//übernimmt den Defaultstyle (oder die Vorlage des Defaultstyles) in einen dynamischen Style, sofern das Zielfeld leer ist
	var obj=document.getElementById(objId);
	if (obj) {
		if (obj.dataset.type<=1) {
			if (obj.value=="") {
				obj.value=newValue;
			}
		} else {
			if (obj.dataset.value=="" || obj.dataset.value=="0") {
				obj.dataset.value=newValue;
			}
		}
	}
}


/*
============================================================================
Kameraarchive (App 102)
============================================================================
*/

function app102_toggle(id,mode,winId) {
	var n=document.getElementById(winId+"-fd4").value;
	if (mode==1) {
		//Mute toggeln
		n=n.replace("2","1");
		if (n.substr(id,1)=="0") {
			n=n.substr(0,id)+"1"+n.substr(id+1);
		} else {
			n=n.substr(0,id)+"0"+n.substr(id+1);
		}
	} else {
		//Solo toggeln
		if (n.substr(id,1)=="2") {
			nn="";
			for (var t=0;t<n.length;t++) {nn+="1";}
			n=nn;
		} else {
			nn="";
			for (var t=0;t<n.length;t++) {nn+="0";}
			n=nn.substr(0,id)+"2"+nn.substr(id+1);
		}
	}
	document.getElementById(winId+"-fd4").value=n;
	ajax("start",102,winId,"",controlGetFormData(winId+"-form1"));
}


/*
============================================================================
Serverzeit nach Clientzeit einstellen (App 1004)
============================================================================
*/

function app1004_refreshDatetime(winId) {
	if (document.getElementById(winId+"-main") && document.getElementById(winId+"-main").dataset.timer) {
		document.getElementById(winId+"-client0").value=getClientDatetime(1);
		document.getElementById(winId+"-client1").value=getClientDatetime(2);
		document.getElementById(winId+"-main").dataset.timer=window.setTimeout(function(){app1004_refreshDatetime(winId);},1000);
	}
}

function app1004_setDatetime(winId) {
	document.getElementById(winId+"-fd0").value=getClientDatetime(1);
	document.getElementById(winId+"-fd1").value=getClientDatetime(2);
	ajax("validateValue","1004",winId,"",controlGetFormData(winId+"-form1"));
}


/*
============================================================================
Projekt laden/duplizieren/löschen (App 104)
============================================================================
*/

function app104_itemSelect(appId,winId,objId,prjId) {
	//Befehle auf die Buttons legen
	document.getElementById(winId+"-list1-cmd1").dataset.cmd='ajaxConfirmSecure("Soll dieses Projekt-Archiv wirklich geöffnet werden? Das aktuelle Projekt geht verloren, falls es nicht archiviert wurde!","loadProject","'+appId+'","'+winId+'","","'+prjId+'");';
	document.getElementById(winId+"-list1-cmd2").dataset.cmd='ajax("renameArchiv","'+appId+'","'+winId+'","","'+prjId+'");';
	document.getElementById(winId+"-list1-cmd3").dataset.cmd='ajaxConfirm("Soll dieses Projekt-Archiv wirklich gelöscht werden?","deleteArchiv","'+appId+'","'+winId+'","","'+prjId+'");';

	//Markierung löschen und auf das aktuelle item setzen
	var tmp=document.getElementById(winId+"-list1").getElementsByTagName("div")
	for (var t=0; t<tmp.length; t++) {
		tmp[t].style.background="";
	}
	document.getElementById(objId).style.background="#80e000";
}


/*
============================================================================
Maskierung bearbeiten (App 1020)
============================================================================
*/

function app1020_drawMask(winId) {
	var maskdata=document.getElementById(winId+"-fd3").value.split(",");
	var n="";
	if (maskdata[1]>0 && maskdata[2]>0) {
		n+="<table width='100%' height='100%' border='1' cellpadding='0' cellspacing='0' style='border:1px solid #a0a0a0; border-collapse:collapse; font-size:1px;'>";
		for (var y=0; y<maskdata[2]; y++) {
			n+="<tr>";
			for (var x=0; x<maskdata[1]; x++) {
				var id=x+(y*maskdata[1]);
				if (maskdata[3].substr(0,1)=="!") {
					if (maskdata[5].substr(id,1)=="1") {
						n+="<td onClick='app1020_toggleMask(\""+winId+"\","+id+",this);' data-value='1' style='background:rgba(80,255,0,0.5);'>&nbsp;</td>";
					} else {
						n+="<td onClick='app1020_toggleMask(\""+winId+"\","+id+",this);' data-value='0' style='background:#"+maskdata[3].substr(1,6)+";'>&nbsp;</td>";
					}
				} else {
					if (maskdata[5].substr(id,1)=="1") {
						n+="<td onClick='app1020_toggleMask(\""+winId+"\","+id+",this);' data-value='1' bgcolor='#"+maskdata[3]+"'>&nbsp;</td>";
					} else {
						n+="<td onClick='app1020_toggleMask(\""+winId+"\","+id+",this);' data-value='0'>&nbsp;</td>";
					}
				}
			}
			n+="</tr>";
		}
		n+="</table>";
	}
	document.getElementById(winId+"-maskedit").innerHTML=n;
}

function app1020_newMask(winId) {
	var mode=document.getElementById(winId+"-fd10").dataset.value;
	var w=document.getElementById(winId+"-fd11").dataset.value;
	var h=document.getElementById(winId+"-fd12").dataset.value;
	var color=document.getElementById(winId+"-fd13").dataset.value;
	var quality=document.getElementById(winId+"-fd14").dataset.value;
	var n="";
	for (var t=0; t<(w*h); t++) {n+="0";}
	document.getElementById(winId+"-fd3").value=mode+","+w+","+h+","+color+","+quality+","+n;
	app1020_drawMask(winId);
}

function app1020_changeMask(winId) {
	var mode=document.getElementById(winId+"-fd10").dataset.value;
	var color=document.getElementById(winId+"-fd13").dataset.value;
	var quality=document.getElementById(winId+"-fd14").dataset.value;
	var maskdata=document.getElementById(winId+"-fd3").value.split(",");
	document.getElementById(winId+"-fd3").value=mode+","+maskdata[1]+","+maskdata[2]+","+color+","+quality+","+maskdata[5];
	app1020_drawMask(winId);
}

function app1020_toggleMask(winId,id,obj) {
	var maskdata=document.getElementById(winId+"-fd3").value.split(",");
	if (obj.dataset.value=="1") {
		if (maskdata[3].substr(0,1)=="!") {obj.style.background="#"+maskdata[3].substr(1,6);} else {obj.style.background="transparent";}
		obj.dataset.value="0";
		maskdata[5]=maskdata[5].substr(0,id)+"0"+maskdata[5].substr(id+1);
	} else {
		if (maskdata[3].substr(0,1)=="!") {obj.style.background="rgba(80,255,0,0.5)";} else {obj.style.background="#"+maskdata[3];}
		obj.dataset.value="1";
		maskdata[5]=maskdata[5].substr(0,id)+"1"+maskdata[5].substr(id+1);
	}
	document.getElementById(winId+"-fd3").value=maskdata[0]+","+maskdata[1]+","+maskdata[2]+","+maskdata[3]+","+maskdata[4]+","+maskdata[5];
}

function app1020_returnMask(winId,destId) {
	//wenn kein Block maskiert ist, wird mask="" gesetzt
	var maskdata=document.getElementById(winId+"-fd3").value.split(",");
	if (maskdata[5].search("1")>=0) {
		document.getElementById(destId).value=document.getElementById(winId+"-fd3").value;
	} else {
		document.getElementById(destId).value="";
	}
	closeWindow(winId);
}


/*
============================================================================
History verwalten (App 9999)
============================================================================
*/

function app9999_historyAdd(objId,helpFn) {
	//### nur machen, wenn der letzte Eintrag nicht schon =helpFn ist!!!
	var item=document.getElementById(objId).dataset.history.split("/");
	if (helpFn.toString()!==item[item.length-1].toString()) {
		document.getElementById(objId).dataset.history+="/"+helpFn;
	}
}

function app9999_historyBack(winId,objId) {
	var item=document.getElementById(objId).dataset.history.split("/");
	if (item.length>1) {
		item.pop();
		document.getElementById(objId).dataset.history=item.join("/");
		ajax("showHelp","9999",winId,item[item.length-1].toString(),"");
	}
}

function app9999_historyHome(winId,objId) {
	document.getElementById(objId).dataset.history="0";
	ajax("showHelp","9999",winId,"0","");
}


/*
============================================================================
Log öffnen/löschen (App 103)
============================================================================
*/

function app103_clickLog(winId,filename,ts) {
	if (!event) var event=window.event;
	if (event.button==0) {
		window.open("../data/log/"+filename+"?"+ts,"_blank");
	}
	if (event.button==2) {
		var n="<div class='contextMenuItem' onClick='controlContextmenuClose(\""+winId+"\"); ajaxConfirm(\"Soll die Logdatei <u>"+filename+"</u> wirklich gelöscht werden?\",\"deleteLog\",\"103\",\""+winId+"\",\"\",\""+filename+"\");'>Löschen</div>";
		n+="<div style='padding:3px;'><div style='widht:100%; height:1px; background:#c0c0c0;'></div></div>";
		n+="<div style='padding:3px; color:#808080;'>"+filename+"</div>";
		var cm=controlContextmenuOpen(winId);
		cm.innerHTML=n;
	}
	clickCancel();
}


/*
============================================================================
Autofocus setzen (für Chrome) (alle Apps)
============================================================================
*/
function appAll_setAutofocus(parentId) {
	var obj=document.getElementById(parentId).querySelector("[autofocus]");
	if (obj) {
		obj.focus();
	}
}
